% This script reads a velocity file created by the code NS3D (velo_rho_vort.t=****.***)
% and that corresponds to a linear simulation around a base state with a single mode.
% This original simulation is made in a physical box of size lx*ly*lz and with a number
% of grid points nx*ny*1
% Then, it creates a new velocity file that corresponds to the addition to the base state to
% the perturbation velocity with a given amplitude and modulated along the z direction with
% a single wavenumber kz_f_f=2*pi/lz_f.
% This new velocity file corresponds to a physical box of size lx*ly*lz_f and
% a number of grid points nx_f*ny_f*nz_f


if strcmp('PCWIN', computer)
	ls('.');
else
	system(['ls -t ', '.']);
end

perturbation_file = input('velocity file to use: ', 's');
amplitude = input('perturbation amplitude: ');
lz_final = input('Hauteur verticale finale: ');
nx_final = input('Nb de points suivant x du fichier final: ');
ny_final = input('Nb de points suivant y du fichier final: ');
nz_final = input('Nb de points suivant z du fichier final: ');

% on commence par essayer de lire le fichier en little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(perturbation_file, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% si le flag est different de 1, on essaye le big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(perturbation_file, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % on verifie que le flag vaut bien 1 cette fois-ci
    if flag ~= 1
        fprintf(1, '\n Il y a un probleme dans le format du fichier : le flag est different de 1\n');
        return
    end
end


% on lit toutes les metadonnees
fprintf('Reading of the velocity file...');

FID = fopen(perturbation_file, 'r',type_endian);
poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

if nz ~= 1
    fprintf(1, '\n Il y a un probleme dans le format du fichier : nz est different de 1\n');
    return
end

% we read the perturbation field
poub = fread(FID, 1, 'integer*4');
instant = fread(FID, 1, 'real*8');
poub = fread(FID, 2, 'integer*4');
Velo_X = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Velo_Y = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Velo_Z = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Rho = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Vort_X = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Vort_Y = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Vort_Z = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 1, 'integer*4');

% we read the base state
poub = fread(FID, 1, 'integer*4');
VBX = fread(FID, nx*ny, 'real*8');
VBY = fread(FID, nx*ny, 'real*8');
VBZ = fread(FID, nx*ny, 'real*8');
poub = fread(FID, 2, 'integer*4');
WBX = fread(FID, nx*ny, 'real*8');
WBY = fread(FID, nx*ny, 'real*8');
WBZ = fread(FID, nx*ny, 'real*8');
status = fclose(FID);
fprintf(' -> ok\n');

% we display some informations on the simulation
fprintf(1, 'Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, 'Physical dimensions of the simulation: %f * %f * %f \n', lx, ly, lz);

Velo_X = reshape(Velo_X, [nx,ny]);
Velo_Y = reshape(Velo_Y, [nx,ny]);
Velo_Z = reshape(Velo_Z, [nx,ny]);
Rho    = reshape(Rho,    [nx,ny]);
Vort_X = reshape(Vort_X, [nx,ny]);
Vort_Y = reshape(Vort_Y, [nx,ny]);
Vort_Z = reshape(Vort_Z, [nx,ny]);

% on reordonne les tableaux
VBX = reshape(VBX, [nx,ny]);
VBY = reshape(VBY, [nx,ny]);
VBZ = reshape(VBZ, [nx,ny]);
WBX = reshape(WBX, [nx,ny]);
WBY = reshape(WBY, [nx,ny]);
WBZ = reshape(WBZ, [nx,ny]);

WBZ = WBZ-2*2*pi/(lx*ly);

fprintf('Determination and writing of the new velocity field...');

% we determine the ratio for the amplitude...
signe = -sign(Vort_Z(1,1));
ratio = amplitude*signe*max(max(abs(WBZ)))/max(max(abs(Vort_Z)));

% original grid
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
[X_m,Y_m] = meshgrid(Y,X);

% final grid
% (we avoid that an interpolation point is outside the range spanned by X,Y,Z).
X_final = linspace(0, lx-lx/nx_final, nx_final);
Y_final = linspace(0, ly-ly/ny_final, ny_final);
X_final = min(X_final,lx-lx/nx);
Y_final = min(Y_final,ly-ly/ny);
[X_final_m,Y_final_m] = meshgrid(Y_final,X_final);

Z_final = linspace(0, lz_final-lz_final/nz_final, nz_final);
kz_f = 2*pi/lz_final;

VBX_m = interp2(X_m,Y_m,VBX,X_final_m,Y_final_m);
VBY_m = interp2(X_m,Y_m,VBY,X_final_m,Y_final_m);
VBZ_m = interp2(X_m,Y_m,VBZ,X_final_m,Y_final_m);
WBX_m = interp2(X_m,Y_m,WBX,X_final_m,Y_final_m);
WBY_m = interp2(X_m,Y_m,WBY,X_final_m,Y_final_m);
WBZ_m = interp2(X_m,Y_m,WBZ,X_final_m,Y_final_m);

Velo_X_m = interp2(X_m,Y_m,Velo_X,X_final_m,Y_final_m);
Velo_Y_m = interp2(X_m,Y_m,Velo_Y,X_final_m,Y_final_m);
Velo_Z_m = interp2(X_m,Y_m,Velo_Z,X_final_m,Y_final_m);
Rho_m    = interp2(X_m,Y_m,Rho,   X_final_m,Y_final_m);
Vort_X_m = interp2(X_m,Y_m,Vort_X,X_final_m,Y_final_m);
Vort_Y_m = interp2(X_m,Y_m,Vort_Y,X_final_m,Y_final_m);
Vort_Z_m = interp2(X_m,Y_m,Vort_Z,X_final_m,Y_final_m);

% we open the final file and we write the different fields
FID = fopen(['initial_velocity_amp=',num2str(amplitude),'_',num2str(nx_final),'x',num2str(ny_final),'x',num2str(nz_final),'_L=', num2str(lx),'x', num2str(ly),'x',num2str(lz_final),'_be'], 'w','ieee-be');

fwrite(FID, 124, 'integer*4');
fwrite(FID, 1, 'integer*4');
fwrite(FID, nx_final, 'integer*4');
fwrite(FID, ny_final, 'integer*4');
fwrite(FID, nz_final, 'integer*4');
fwrite(FID, lx, 'real*8');
fwrite(FID, ly, 'real*8');
fwrite(FID, lz_final, 'real*8');
fwrite(FID, dt, 'real*8');
fwrite(FID, truncate, 'integer*4');
fwrite(FID, type_trunc, 'integer*4');
fwrite(FID, rtrunc_x, 'real*8');
fwrite(FID, rtrunc_y, 'real*8');
fwrite(FID, rtrunc_z, 'real*8');
fwrite(FID, nu, 'real*8');
fwrite(FID, stratification, 'integer*4');
fwrite(FID, xns, 'real*8');
fwrite(FID, schm, 'real*8');
fwrite(FID, omega2, 'real*8');
fwrite(FID, perturb, 'integer*4');
fwrite(FID, lin, 'integer*4');
fwrite(FID, 124, 'integer*4');

fwrite(FID, 8, 'integer*4');
fwrite(FID, 0., 'real*8');
fwrite(FID, 8, 'integer*4');



% we determine and we write the new velocity and vorticity fields

% Velo_X
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');
for iz=1:nz_final
	fwrite(FID, VBX_m + ratio*Velo_X_m*cos(kz_f*Z_final(iz)), 'real*8');
end
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');


% Velo_Y
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');
for iz=1:nz_final
	fwrite(FID, VBY_m + ratio*Velo_Y_m*cos(kz_f*Z_final(iz)), 'real*8');
end
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');


% Velo_Z
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');
for iz=1:nz_final
	fwrite(FID, VBZ_m + ratio*Velo_Z_m*sin(-kz_f*Z_final(iz)), 'real*8');
end
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');


% Rho
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');
for iz=1:nz_final
	fwrite(FID, zeros(nx_final,ny_final), 'real*8');
end
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');


% Vort_X
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');
for iz=1:nz_final
	fwrite(FID, WBX_m + ratio*Vort_X_m*sin(-kz_f*Z_final(iz)), 'real*8');
end
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');


% Vort_Y
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');
for iz=1:nz_final
	fwrite(FID, WBY_m + ratio*Vort_Y_m*sin(-kz_f*Z_final(iz)), 'real*8');
end
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');


% Vort_Z
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');
for iz=1:nz_final
	fwrite(FID, WBZ_m + ratio*Vort_Z_m*cos(kz_f*Z_final(iz)), 'real*8');
end
fwrite(FID, nx_final*ny_final*nz_final*8, 'integer*4');


% Base state
fwrite(FID, nx_final*ny_final*8*3, 'integer*4');
fwrite(FID, VBX_m, 'real*8');
fwrite(FID, VBY_m, 'real*8');
fwrite(FID, VBZ_m, 'real*8');
fwrite(FID, nx_final*ny_final*8*3, 'integer*4');

fwrite(FID, nx_final*ny_final*8*3, 'integer*4');
fwrite(FID, WBX_m, 'real*8');
fwrite(FID, WBY_m, 'real*8');
fwrite(FID, WBZ_m, 'real*8');
fwrite(FID, nx_final*ny_final*8*3, 'integer*4');

status = fclose(FID);

fprintf(' -> ok\n');
