program io_stride

    implicit none

        integer, parameter :: nx=512
        integer, parameter :: ny=512
        integer, parameter :: nz=8
        integer, parameter :: str_x=2
        integer, parameter :: str_y=2
        integer, parameter :: str_z=2

        ! we determine the new data size
        integer(kind=4), parameter :: nxp = 1 + (nx-1)/str_x, nyp = 1 + (ny-1)/str_y, nzp = 1 + (nz-1)/str_z

        double precision, dimension(nx,ny,nz) :: a
        double precision, dimension(nxp,nyp,nzp) :: b

        double precision, dimension(nx,ny) :: c1,c2,c3
        double precision, dimension(nxp,nyp) :: d1,d2,d3

        integer(kind=4) :: nxread, nyread, nzread, flag, type_trunc
        double precision :: lx, ly, lz, dt, rtrunc_x, rtrunc_y, rtrunc_z, nu, xns, schm, omega2, instant
        logical(kind=4) :: de_aliasing, stratification, perturbative, linear

        integer :: trash_int
        integer :: i_field

        character(len=200) :: file_name

        write(*,*) 'Initial dimensions: ', nx, ny, nz
        write(*,*) 'Reduced dimensions: ', nxp, nyp, nzp

        write(*,*) 'Name of the velocity file:'
        read(*,*) file_name
        call stride_array(file_name)

        contains

        subroutine stride_array(file_name)

            implicit none

            character(len=200), intent(in) :: file_name
						integer(kind=4) :: temp_int
						
            ! we open the input and output files

            ! correction des headers
            temp_int=124
            open (unit=8,file=file_name,form='unformatted',access='direct',recl=4)
						write(8,rec=1) temp_int
						close(8)
						
						open (unit=8,file=file_name,form='unformatted',access='direct',recl=4)
						write(8,rec=33) temp_int
						close(8)
						
						temp_int=8
						open (unit=8,file=file_name,form='unformatted',access='direct',recl=4)
						write(8,rec=34) temp_int
						close(8)
						
						open (unit=8,file=file_name,form='unformatted',access='direct',recl=4)
						write(8,rec=37) temp_int
						close(8)

           
             ! metadata
            open (unit=8,file=file_name,form='unformatted')
            read(8) flag, nxread, nyread, nzread, lx, ly, lz, dt, de_aliasing, type_trunc, rtrunc_x, rtrunc_y, rtrunc_z, &
                      nu, stratification, xns, schm, omega2, perturbative, linear
            read(8) instant
            close(8)

            open (unit=10,file='stride_'//file_name,form='unformatted')
            write(10) flag, nxp, nyp, nzp, lx, ly, lz, dt, de_aliasing, type_trunc, rtrunc_x, rtrunc_y, rtrunc_z, &
                      nu, stratification, xns, schm, omega2, perturbative, linear
            write(10) instant


            ! we read, stride and write the different fields of the data file
            do i_field=7,7
                call io_array_3d(i_field)
            end do

            call io_array_2d(1)
            call io_array_2d(2)

            close(10)

        end subroutine


        ! read, stride and write a 3D field (velocity, density, vorticity)
        subroutine io_array_3d(type_field)

            implicit none

            integer, intent(in) :: type_field
            double precision, dimension(19) :: trash_header
            integer :: ix, iy, iz, position_record

            if (type_field==1) then
                position_record= 33*4 + (4+8+4) + 4 + (4+8*nx*ny*nz+4)
                open (unit=8,file=file_name,access='direct',form='unformatted',recl=position_record)
                read(8,rec=1) trash_header, a
                close(8)
            else
                position_record= 33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4
                open (unit=8,file=file_name,access='direct',form='unformatted',recl=position_record)
                read(8,rec=2) a
                close(8)
            end if

            do iz=1, nzp
                do iy=1, nyp
                    do ix=1, nxp
                        b(ix, iy, iz) = a( (ix-1)*str_x +1, (iy-1)*str_y +1, (iz-1)*str_z +1)
                    end do
                end do
            end do

            write(10) b

        end subroutine

        ! read, stride and write the base field
        subroutine io_array_2d(type_field)

            implicit none

            integer, intent(in) :: type_field
            integer :: ix, iy, position_record

            position_record= 33*4 + (4+8+4) + (8-1)*(4+8*nx*ny*nz+4) + (type_field-1)*(4+3*8*nx*ny+4) + 4
            open (unit=8,file=file_name,access='direct',form='unformatted',recl=position_record)
            read(8,rec=2) c1,c2,c3
            close(8)

            do iy=1, nyp
                do ix=1, nxp
                    d1(ix, iy) = c1( (ix-1)*str_x +1, (iy-1)*str_y +1)
                    d2(ix, iy) = c2( (ix-1)*str_x +1, (iy-1)*str_y +1)
                    d3(ix, iy) = c3( (ix-1)*str_x +1, (iy-1)*str_y +1)
                end do
            end do

            write(10) d1,d2,d3

        end subroutine

end program io_stride
