% ce script permet de tracer des plans de coupe dans le plan xz

% ######################
% global_vars a renseigner
% ######################

% field to plot (vx=1,vy=2,vz=3,rho=4,wx=5,wy=6,wz=7)
type_field=4;


% ######################
% on touche plus
% ######################

if strcmp('PCWIN', computer)
	system('dir /O-D /AD /B .');
else
	system('ls -t .');
end

fichier = input('\nFichier de vitesse a representer: ', 's');
fichier_derivative = input('Vertical derivatives file to use: ', 's');

% we determine the type of binary data (little-endian, big-endian) with the velocity file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

xnsqr = xns^2;

status = fclose(FID);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear\n'];
    else
        texte = ['perturbative/linear\n'];
    end
end

% on cree la grille des points de colocation
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);


% on demande quel plan representer
ny0 = floor(input('\nPlan de coupe: '));


%%%%%%%%%%%   On lit le champ a representer  %%%%%%%%%%%
fprintf(1, '      Reading field...');
FID = fopen(fichier,'r',type_endian);

% on lit toutes les metadonnees d'un coup
fseek(FID, 4*34, 'bof');
instant = fread(FID, 1, 'real*8');

% on va directement lire le champ en n'enregistrant que les valeurs appartenant au plan xz voulu
field = zeros(nx,nz);
for iz=1:nz
	fseek(FID,33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny + 8*nx*(ny0-1), 'bof');
	field(:,iz) = fread(FID, nx, 'real*8');
end


%%%%%%%%%%%   On lit les derivees par rapport a z  %%%%%%%%%%%
fprintf(1, '\n      Reading derivatives...');
FID = fopen(fichier_derivative,'r',type_endian);

% on lit toutes les metadonnees d'un coup
fseek(FID, 4*17, 'bof');
instant = fread(FID, 1, 'real*8');


% on va directement lire le champ en n'enregistrant que les valeurs appartenant au plan xz voulu
for iz=1:nz
    type_field=1;
	fseek(FID,16*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny + 8*nx*(ny0-1), 'bof');
	dudz(:,iz) = fread(FID, nx, 'real*8');
	
	type_field=2;
	fseek(FID,16*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny + 8*nx*(ny0-1), 'bof');
	dvdz(:,iz) = fread(FID, nx, 'real*8');
	
	type_field=3;
	fseek(FID,16*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny + 8*nx*(ny0-1), 'bof');
	drhodz(:,iz) = fread(FID, nx, 'real*8');
end

% On calcule Ri
Rix = xnsqr*dudz.^(-2);
Riy = xnsqr*dvdz.^(-2);
Rit = xnsqr*(dudz.^2+dvdz.^2).^-1;

fprintf(1, '\n      Plotting...\n');

clf

subplot(4,1,1,'align')
hold on
pcolor(X,Z,field');
axis equal tight   
title(['b''   t=', num2str(instant)]);   
shading interp
lighting phong
set(gca,'LineWidth',2,'XTick',[0 0.5 1 1.5 2], 'YTick',[0 0.2 0.4])
%contour(X,Z,Rit',0.25,'k');
%caxis([-0.03 0.03])
colorbar
box on

subplot(4,1,2,'align')
hold on
pcolor(X,Z,Rit');
axis equal tight    
title(['Ri_T   t=', num2str(instant)]);
shading interp
lighting phong
set(gca,'LineWidth',2,'XTick',[0 0.5 1 1.5 2], 'YTick',[0 0.2 0.4])
contour(X,Z,Rit',0.25,'k');
caxis([0 1]);
colorbar;
box on

subplot(4,1,3,'align')
hold on
pcolor(X,Z,(1-drhodz)');
axis equal tight    
title(['1-db/dz   t=', num2str(instant), '     min value=', num2str(min(min((1-drhodz))))]);
shading interp
lighting phong
set(gca,'LineWidth',2,'XTick',[0 0.5 1 1.5 2], 'YTick',[0 0.2 0.4])
contour(X,Z,Rit',0.25,'k');
caxis([-1 1]);
colorbar;
box on

subplot(4,1,4,'align')
hold on
pcolor(X,Z,field'-Z'*ones(1,nx));
axis equal tight    
title(['b''-z   t=', num2str(instant)]);
shading interp
lighting phong
set(gca,'LineWidth',2,'XTick',[0 0.5 1 1.5 2], 'YTick',[0 0.2 0.4])
contour(X,Z,Rit',0.25,'k');
colorbar;
box on


!mkdir -p crosscuts_matlab
orient landscape
set(gcf,'PaperType','a4')
%print('-dpng','-r600',['crosscuts_matlab/Rit_rho.t=',num2str(instant),'.png']);
saveas(gcf,['crosscuts_matlab/Rit_rho.t=',num2str(instant),'.fig'],'fig')
