program rawdata

    implicit none

    integer, parameter :: nx=768
    integer, parameter :: ny=768
    integer, parameter :: nz=96
    integer :: type_field=7

    character(len=200) :: file_name
    character(len=10)  :: type_field_name

    if (type_field==1) then
   		type_field_name='vx'
    else if (type_field==2) then
    		type_field_name='vy'
    else if (type_field==3) then
    		type_field_name='vz'
    else if (type_field==4) then
    		type_field_name='rho'
    else if (type_field==5) then
    		type_field_name='wx'
    else if (type_field==6) then
    		type_field_name='wy'
    else if (type_field==7) then
    		type_field_name='wz'
   	end if

    write(*,*)
    write(*,'(A24,TR3,I5,TR1,A1,TR1,I5,TR1,A1,TR1,I5)') ' Dimensions of the data:', nx,'x',ny,'x',nz
    write(*,*) 'Enter the name of the velocity file:'
    read(*,*) file_name


    write(*,*) 'File being processed:  ', trim(file_name)
    call convert_raw(file_name,type_field)
    write(*,*) '    --> ok'


    contains


    subroutine convert_raw(file_name,type_field)

        implicit none

        character(len=200), intent(in) :: file_name
        integer, intent(in) :: type_field

        double precision, dimension(nx,ny), save :: data_field
        integer :: position_record
        integer(kind=4) :: flag, nxread, nyread, nzread
        double precision :: instant
        character(len=10) :: string_time
        integer :: iz, iz_local
	    integer(kind=4), dimension(31) :: trash_header
        double precision :: trash_double

	    ! we check the dimensions
        open (unit=8,file=file_name,form='unformatted')
        read(8) flag,nxread,nyread,nzread
        close(8)
        if (flag /= 1 .or. nxread /= nx .or. nyread /= ny .or. nzread /= nz) then
            write(*,*) "ERROR: the data file has bad format. Failure."
            stop
        end if

        ! we retrieve the time of the record
        open (unit=8,file=file_name,access='direct',form='unformatted',recl=4*34)
        read(8,rec=2) instant
        close(8)
		call time2string(instant,string_time)
		write(*,*) '    file internal time:  ', instant

        do iz=1,nz
            position_record= 33*4 + (4+8+4) + (type_field-1)*(4+8*nx*ny*nz+4) + 4 + 8*(iz-1)*nx*ny
            
		if (position_record >= nx*ny) then
			open (unit=8,file=file_name,access='direct',form='unformatted',recl=position_record)
      	      read(8,rec=2) data_field
            	close(8)
		else
			open (unit=8,file=file_name,form='unformatted')
			read(8) trash_header
			read(8) trash_double
			do iz_local=1,iz
				read(8) data_field
			end do
			close(8)
		end if

            open (unit=10,file=trim(type_field_name)//'_'//string_time//'.raw',form='unformatted', &
                  access='direct', recl=4*nx*ny)
            write(10,rec=iz) real(data_field)
            close(10)
        end do

    end subroutine


! *************************************************************************
    subroutine time2string(time,string_time)
! *************************************************************************
    !****f* subfunctions/time2string
    !  NAME
    !    time2string
    !
    !  DESCRIPTION
    !    Convert time to a formatted string like "t=0000.000"
    !
    !  INPUTS
    !    time: time to convert in a string
    !
    !  OUTPUT
    !    string_time: string with the desired format
    !***

        implicit none

        double precision, intent(in) :: time
        character(len=10), intent(out) :: string_time

        write(string_time,'(A2,I4.4,A1,I3.3)') 't=',int(time),'.',nint(1000*(time-int(time)))
  end subroutine

end program rawdata
