% This script reads a velocity file created by the code NS3D (velo_rho_vort.t=****.***)
% and that corresponds to a physical box of size lx*ly*lz and a number of grid points nx*ny*nz.
% Then, it creates a new velocity file interpolated from the original one and that corresponds
% to a new physical box of size lx_f*ly_f*lz_f and a number of grid points nx_f*ny_f*nz_f


if strcmp('PCWIN', computer)
	ls('.');
else
	system(['ls -t ', '.']);
end

name_file = input('velocity file to use: ', 's');

% on commence par essayer en little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(name_file, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% si le flag est different de 1, on essaye le big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(name_file, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % on verifie que le flag vaut bien 1 cette fois-ci
    if flag ~= 1
        fprintf(1, '\n Il y a un probleme dans le format du fichier : le flag est different de 1\n');
        return
    end
end


% on lit toutes les metadonnees
fprintf('Reading of the velocity file...');

FID = fopen(name_file, 'r',type_endian);
poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

poub            = fread(FID, 1, 'integer*4');
instant         = fread(FID, 1, 'real*8');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);


% we display some informations on the simulation and ask for the output dimensions
fprintf(1, 'Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, 'Physical dimensions of the simulation: %f * %f * %f \n', lx, ly, lz);

% we ask for the final dimensions of the box
lx_f = input('Nouvelle taille physique de la boite suivant x: ');
ly_f =  input('Nouvelle taille physique de la boite suivant y: ');
lz_f = input('Nouvelle taille physique de la boite suivant z: ');

x_min = input('Valeur de depart suivant x : ');
y_min = input('Valeur de depart suivant y : ');
z_min = input('Valeur de depart suivant z : ');

nx_f = input('Nb de points suivant x du fichier final: ');
ny_f = input('Nb de points suivant y du fichier final: ');
nz_f = input('Nb de points suivant z du fichier final: ');

% nb_etape = input('Nb d etapes lors de l interpolation: ');
nb_etape = nz_f;


% we open the destination file and write the header
FID2 = fopen([name_file, '_', num2str(nx_f), 'x', num2str(ny_f), 'x', num2str(nz_f), '_L=', num2str(lx_f), 'x', num2str(ly_f), 'x', num2str(lz_f)], 'w','ieee-be');

fwrite(FID2, 124, 'integer*4');
fwrite(FID2, 1, 'integer*4');
fwrite(FID2, nx_f, 'integer*4');
fwrite(FID2, ny_f, 'integer*4');
fwrite(FID2, nz_f, 'integer*4');
fwrite(FID2, lx_f, 'real*8');
fwrite(FID2, ly_f, 'real*8');
fwrite(FID2, lz_f, 'real*8');
fwrite(FID2, dt, 'real*8');
fwrite(FID2, truncate, 'integer*4');
fwrite(FID2, type_trunc, 'integer*4');
fwrite(FID2, rtrunc_x, 'real*8');
fwrite(FID2, rtrunc_y, 'real*8');
fwrite(FID2, rtrunc_z, 'real*8');
fwrite(FID2, nu, 'real*8');
fwrite(FID2, stratification, 'integer*4');
fwrite(FID2, xns, 'real*8');
fwrite(FID2, schm, 'real*8');
fwrite(FID2, omega2, 'real*8');
fwrite(FID2, perturb, 'integer*4');
fwrite(FID2, lin, 'integer*4');
fwrite(FID2, 124, 'integer*4');

fwrite(FID2, 8, 'integer*4');
fwrite(FID2, instant, 'real*8');
fwrite(FID2, 8, 'integer*4');

% original grid
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);

% final grid (we avoid to have values outside the range spanned by X,Y,Z)
x_max = x_min + lx_f;
y_max = y_min + ly_f;
z_max = z_min + lz_f;

X_f = linspace(x_min, x_max-x_max/nx_f, nx_f);
Y_f = linspace(y_min, y_max-y_max/ny_f, ny_f);
Z_f = linspace(z_min, z_max-z_max/nz_f, nz_f);

X_f = min(X_f,lx-lx/nx);
Y_f = min(Y_f,ly-ly/ny);
Z_f = min(Z_f,lz-lz/nz);


%%%%%%%%%%%%%%%%%%%%%%
% VELOCITY, DENSITY AND VORTICITY
%%%%%%%%%%%%%%%%%%%%%%

fprintf(1, 'Interpolation of the velocity, density and vorticity...\n');

% we determine the indices corresponding to the different steps (we split
% the data in the z- direction)
ind_etape_f = floor(linspace(0,nz_f,nb_etape+1));

% we loop on each "big" field (vx, vy, vz, rho, wx, wy, wz)
for k=1:7

    fprintf(1, '\n  Field %i...\n', k);

    % we write the dimension of the field in bytes before writing the field
    fwrite(FID2, nx_f*ny_f*nz_f*8, 'integer*4');

    for i=1:nb_etape
        fprintf(1, '       etape %i/%i...\n', i, nb_etape);
        ind_debut_f = ind_etape_f(i)+1;
        ind_fin_f = ind_etape_f(i+1);

        % we get the minimum range of indices necessary in the z-direction
        % for the interpolation

        [trash,ind_debut] = min(Z<=Z_f(ind_debut_f));
        if (trash==1)
            ind_debut = nz;
        else
            ind_debut = ind_debut - 1;
        end

        [trash,ind_fin] = min(Z<Z_f(ind_fin_f));
        if (trash==1)
            ind_fin = nz;
        end

        % on verifie que l'on a fait les bons choix d'indice parce que le risque que je me sois
        % trompe dans les formules precedentes n est pas nul...
        if ( Z_f(ind_debut_f) < Z(ind_debut) |  Z_f(ind_fin_f) > Z(ind_fin))
            fprintf(' -> Il y a un probleme dans les indices d etape...\n');
            stop
        end


        % we read the necessary data in the original field
        FID = fopen(name_file, 'r',type_endian);
        fseek(FID, 33*4 + 4+8+4 +(k-1)*(4+8*nx*ny*nz+4) + 4 +8*(ind_debut-1)*nx*ny, 'bof');
        field = fread(FID, nx*ny*(ind_fin-ind_debut+1), 'real*8');
        status = fclose(FID);

        field = reshape(field, [nx,ny,ind_fin-ind_debut+1]);

        % we create the grids and we interpolate the data
        [Xm,Ym,Zm_etape]       = meshgrid(Y,X,Z(ind_debut:ind_fin));
        [Xm_f,Ym_f,Zm_etape_f] = meshgrid(Y_f,X_f,Z_f(ind_debut_f:ind_fin_f));

        if (ind_fin==ind_debut)
		field_interp = interp2(Xm,Ym,field,Xm_f,Ym_f);
	else
        	field_interp = interp3(Xm,Ym,Zm_etape,field,Xm_f,Ym_f,Zm_etape_f);
	end

	fwrite(FID2, field_interp, 'real*8');

    end
    fwrite(FID2, nx_f*ny_f*nz_f*8, 'integer*4');
end
clear field field_interp Xm Ym Zm_etape Xm_f Ym_f Zm_etape_f;
fprintf(' -> ok\n');


%%%%%%%%%%%%%%%%
% BASIC STATE
%%%%%%%%%%%%%%%%

fprintf(1, 'Interpolation of the basic state...\n');

% we read the base state
FID = fopen(name_file, 'r',type_endian);
fseek(FID, 33*4 + 4+8+4 +7*(4+8*nx*ny*nz+4), 'bof');

poub = fread(FID, 1, 'integer*4');
VBX = fread(FID, nx*ny, 'real*8');
VBY = fread(FID, nx*ny, 'real*8');
VBZ = fread(FID, nx*ny, 'real*8');
poub = fread(FID, 2, 'integer*4');
WBX = fread(FID, nx*ny, 'real*8');
WBY = fread(FID, nx*ny, 'real*8');
WBZ = fread(FID, nx*ny, 'real*8');

VBX = reshape(VBX, [nx,ny]);
VBY = reshape(VBY, [nx,ny]);
VBZ = reshape(VBZ, [nx,ny]);
WBX = reshape(WBX, [nx,ny]);
WBY = reshape(WBY, [nx,ny]);
WBZ = reshape(WBZ, [nx,ny]);


% we interpolate the basic state
[Xm,Ym] = meshgrid(Y,X);
[Xm_f,Ym_f] = meshgrid(Y_f,X_f);

VBX_interp = interp2(Xm,Ym,VBX,Xm_f,Ym_f);
VBY_interp = interp2(Xm,Ym,VBY,Xm_f,Ym_f);
VBZ_interp = interp2(Xm,Ym,VBZ,Xm_f,Ym_f);
WBX_interp = interp2(Xm,Ym,WBX,Xm_f,Ym_f);
WBY_interp = interp2(Xm,Ym,WBY,Xm_f,Ym_f);
WBZ_interp = interp2(Xm,Ym,WBZ,Xm_f,Ym_f);

% we write the basic state
fwrite(FID2, nx_f*ny_f*8*3, 'integer*4');
fwrite(FID2, VBX_interp, 'real*8');
fwrite(FID2, VBY_interp, 'real*8');
fwrite(FID2, VBZ_interp, 'real*8');
fwrite(FID2, nx_f*ny_f*8*3, 'integer*4');

fwrite(FID2, nx_f*ny_f*8*3, 'integer*4');
fwrite(FID2, WBX_interp, 'real*8');
fwrite(FID2, WBY_interp, 'real*8');
fwrite(FID2, WBZ_interp, 'real*8');
fwrite(FID2, nx_f*ny_f*8*3, 'integer*4');

status = fclose(FID);
status = fclose(FID2);

fprintf(' -> ok\n');
