% This script interpolates a data file (velocity, vorticity, density) on a different mesh.


if strcmp('PCWIN', computer)
	ls('.');
else
	system(['ls -t ', '.']);
end

perturbation_file = input('3D perturbation field to use: ', 's');

% on commence par essayer en little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(perturbation_file, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% si le flag est different de 1, on essaye le big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(perturbation_file, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % on verifie que le flag vaut bien 1 cette fois-ci
    if flag ~= 1
        fprintf(1, '\n Il y a un probleme dans le format du fichier : le flag est different de 1\n');
        return
    end
end


% on lit toutes les metadonnees
fprintf('Reading of the velocity file...');

FID = fopen(perturbation_file, 'r',type_endian);
poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb        = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');
poub = fread(FID, 1, 'integer*4');
instant = fread(FID, 1, 'real*8');
poub = fread(FID, 1, 'integer*4');

% we display some informations on the simulation and ask for the output dimensions
fprintf(1, 'Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, 'Physical dimensions of the simulation: %f * %f * %f \n', lx, ly, lz);

nx_final = input('Nb de points suivant x du fichier final: ');
ny_final = input('Nb de points suivant y du fichier final: ');
nz_final = input('Nb de points suivant z du fichier final: ');
nb_etape = input('Nb d etapes lors de l interpolation: ');


% we open the destination file and write the header
FID2 = fopen([perturbation_file, '_', num2str(nx_final), 'x', num2str(ny_final), 'x', num2str(nz_final)], 'w','ieee-be');

fwrite(FID2, 124, 'integer*4');
fwrite(FID2, 1, 'integer*4');
fwrite(FID2, nx_final, 'integer*4');
fwrite(FID2, ny_final, 'integer*4');
fwrite(FID2, nz_final, 'integer*4');
fwrite(FID2, lx, 'real*8');
fwrite(FID2, ly, 'real*8');
fwrite(FID2, lz, 'real*8');
fwrite(FID2, dt, 'real*8');
fwrite(FID2, truncate, 'integer*4');
fwrite(FID2, type_trunc, 'integer*4');
fwrite(FID2, rtrunc_x, 'real*8');
fwrite(FID2, rtrunc_y, 'real*8');
fwrite(FID2, rtrunc_z, 'real*8');
fwrite(FID2, nu, 'real*8');
fwrite(FID2, stratification, 'integer*4');
fwrite(FID2, xns, 'real*8');
fwrite(FID2, schm, 'real*8');
fwrite(FID2, omega2, 'real*8');
fwrite(FID2, perturb, 'integer*4');
fwrite(FID2, lin, 'integer*4');
fwrite(FID2, 124, 'integer*4');

fwrite(FID2, 8, 'integer*4');
fwrite(FID2, 0., 'real*8');
fwrite(FID2, 8, 'integer*4');



% grids necessary for the interpolation
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);
[Xm,Ym,Zm] = meshgrid(X,Y,Z);

XI = linspace(0, lx-lx/nx_final, nx_final);
YI = linspace(0, ly-ly/ny_final, ny_final);
ZI = linspace(0, lz-lz/nz_final, nz_final);

% we avoid that an interpolation point is outside the range spanned by X,Y,Z.
% This would return a "NaN" with the interpolation Matlab functions.
% the error is very small
XI = min(XI,lx-lx/nx);
YI = min(YI,ly-ly/ny);
ZI = min(ZI,lz-lz/nz);

ind_etape = floor(linspace(0,nz_final,nb_etape));


%%%%%%%%%%%%%%%%%%%%%%
% VELOCITY, DENSITY AND VORTICITY
%%%%%%%%%%%%%%%%%%%%%%

fprintf(1, 'Interpolation of the velocity, density and vorticity...\n');

for k=1:7
    fprintf(1, '  field %i...\n', k);

    poub = fread(FID, 1, 'integer*4');
    field = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 1, 'integer*4');

    field = reshape(field, [nx,ny,nz]);

    fwrite(FID2, nx_final*ny_final*nz_final*8, 'integer*4');
    for i=1:nb_etape-1
        ind_debut = ind_etape(i)+1;
        ind_fin = ind_etape(i+1);

        [XIm,YIm,ZIm_etape] = meshgrid(XI,YI,ZI(ind_debut:ind_fin));
        field_interp = interp3(Xm,Ym,Zm,field,XIm,YIm,ZIm_etape);

        fwrite(FID2, field_interp, 'real*8');
    end
    fwrite(FID2, nx_final*ny_final*nz_final*8, 'integer*4');
end
clear field field_interp;
fprintf(' -> ok\n');


%%%%%%%%%%%%%%%%
% BASIC STATE
%%%%%%%%%%%%%%%%

fprintf(1, 'Interpolation of the basic state...\n');

% we read the base state
poub = fread(FID, 1, 'integer*4');
VBX = fread(FID, nx*ny, 'real*8');
VBY = fread(FID, nx*ny, 'real*8');
VBZ = fread(FID, nx*ny, 'real*8');
poub = fread(FID, 2, 'integer*4');
WBX = fread(FID, nx*ny, 'real*8');
WBY = fread(FID, nx*ny, 'real*8');
WBZ = fread(FID, nx*ny, 'real*8');

VBX = reshape(VBX, [nx,ny]);
VBY = reshape(VBY, [nx,ny]);
VBZ = reshape(VBZ, [nx,ny]);
WBX = reshape(WBX, [nx,ny]);
WBY = reshape(WBY, [nx,ny]);
WBZ = reshape(WBZ, [nx,ny]);


% we interpolate the basic state
[Xm,Ym] = meshgrid(X,Y);
[XIm,YIm] = meshgrid(XI,YI);

VBX_interp = interp2(Xm,Ym,VBX,XIm,YIm);
VBY_interp = interp2(Xm,Ym,VBY,XIm,YIm);
VBZ_interp = interp2(Xm,Ym,VBZ,XIm,YIm);
WBX_interp = interp2(Xm,Ym,WBX,XIm,YIm);
WBY_interp = interp2(Xm,Ym,WBY,XIm,YIm);
WBZ_interp = interp2(Xm,Ym,WBZ,XIm,YIm);

% we write the basic state
fwrite(FID2, nx_final*ny_final*8*3, 'integer*4');
fwrite(FID2, VBX_interp, 'real*8');
fwrite(FID2, VBY_interp, 'real*8');
fwrite(FID2, VBZ_interp, 'real*8');
fwrite(FID2, nx_final*ny_final*8*3, 'integer*4');

fwrite(FID2, nx_final*ny_final*8*3, 'integer*4');
fwrite(FID2, WBX_interp, 'real*8');
fwrite(FID2, WBY_interp, 'real*8');
fwrite(FID2, WBZ_interp, 'real*8');
fwrite(FID2, nx_final*ny_final*8*3, 'integer*4');

status = fclose(FID);
status = fclose(FID2);

fprintf(' -> ok\n');
