% This script builds an initial velocity field of size nx*ny*nz used by the NS3D code.
% This initial velocity field is the sum of a base field VB of size nx*ny and
% a velocity field V of size nx*ny*1. This velocity field must come from a linear perturbative
% simulation around the base field VB made with a single mode kz (this is an option of the NS3D code).

% Input :
% a velocity field "velo_rho_vort.t=****.***": it contains a velocity field V and the base state VB.
% It is the result of a linear perturbative simulation around VB made with a single mode kz
%

if strcmp('PCWIN', computer)
	ls('.');
else
	system(['ls -t ', '.']);
end

perturbation_file = input('velocity file to use: ', 's');
amplitude = input('perturbation amplitude: ');
nz_final = input('nombre de modes en z: ');

% on commence par essayer en little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(perturbation_file, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% si le flag est different de 1, on essaye le big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(perturbation_file, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % on verifie que le flag vaut bien 1 cette fois-ci
    if flag ~= 1
        fprintf(1, '\n Il y a un probleme dans le format du fichier : le flag est different de 1\n');
        return
    end
end


% on lit toutes les metadonnees
fprintf('Reading of the velocity file...');

FID = fopen(perturbation_file, 'r',type_endian);
poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb         = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

if nz ~= 1
    fprintf(1, '\n Il y a un probleme dans le format du fichier : nz est different de 1\n');
    return
end

% we read the perturbation field
poub = fread(FID, 1, 'integer*4');
instant = fread(FID, 1, 'real*8');
poub = fread(FID, 2, 'integer*4');
Velo_X = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Velo_Y = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Velo_Z = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Rho = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Vort_X = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Vort_Y = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 2, 'integer*4');
Vort_Z = fread(FID, nx*ny*nz, 'real*8');
poub = fread(FID, 1, 'integer*4');

% we read the base state
poub = fread(FID, 1, 'integer*4');
VBX = fread(FID, nx*ny, 'real*8');
VBY = fread(FID, nx*ny, 'real*8');
VBZ = fread(FID, nx*ny, 'real*8');
poub = fread(FID, 2, 'integer*4');
WBX = fread(FID, nx*ny, 'real*8');
WBY = fread(FID, nx*ny, 'real*8');
WBZ = fread(FID, nx*ny, 'real*8');
status = fclose(FID);
fprintf(' -> ok\n');

% we display some informations on the simulation
fprintf(1, 'Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, 'Physical dimensions of the simulation: %f * %f * %f \n', lx, ly, lz);

Velo_X = reshape(Velo_X, [nx,ny,nz]);
Velo_Y = reshape(Velo_Y, [nx,ny,nz]);
Velo_Z = reshape(Velo_Z, [nx,ny,nz]);
Rho    = reshape(Rho,    [nx,ny,nz]);
Vort_X = reshape(Vort_X, [nx,ny,nz]);
Vort_Y = reshape(Vort_Y, [nx,ny,nz]);
Vort_Z = reshape(Vort_Z, [nx,ny,nz]);

% on reordonne les tableaux
VBX = reshape(VBX, [nx,ny]);
VBY = reshape(VBY, [nx,ny]);
VBZ = reshape(VBZ, [nx,ny]);
WBX = reshape(WBX, [nx,ny]);
WBY = reshape(WBY, [nx,ny]);
WBZ = reshape(WBZ, [nx,ny]);


% we determine the ratio for the amplitude...
fprintf('Determination and writing of the new velocity field...');

ZZ = linspace(0, (nz_final-1)*lz/nz_final, nz_final);
KZ = 2*pi/lz;
ratio = amplitude*max(max(abs(WBZ)))/max(max(max(abs(Vort_Z))));
field_write = zeros(nx,ny,nz_final);

FID = fopen(['initial_velocity_amp=',num2str(amplitude),'_',num2str(nx),'x',num2str(ny),'x',num2str(nz_final),'_L=', num2str(lx),'x', num2str(ly),'x',num2str(lz),'_be'], 'w','ieee-be');

fwrite(FID, 124, 'integer*4');
fwrite(FID, 1, 'integer*4');
fwrite(FID, nx, 'integer*4');
fwrite(FID, ny, 'integer*4');
fwrite(FID, nz_final, 'integer*4');
fwrite(FID, lx, 'real*8');
fwrite(FID, ly, 'real*8');
fwrite(FID, lz, 'real*8');
fwrite(FID, dt, 'real*8');
fwrite(FID, truncate, 'integer*4');
fwrite(FID, type_trunc, 'integer*4');
fwrite(FID, rtrunc_x, 'real*8');
fwrite(FID, rtrunc_y, 'real*8');
fwrite(FID, rtrunc_z, 'real*8');
fwrite(FID, nu, 'real*8');
fwrite(FID, stratification, 'integer*4');
fwrite(FID, xns, 'real*8');
fwrite(FID, schm, 'real*8');
fwrite(FID, omega2, 'real*8');
fwrite(FID, perturb, 'integer*4');
fwrite(FID, lin, 'integer*4');
fwrite(FID, 124, 'integer*4');

fwrite(FID, 8, 'integer*4');
fwrite(FID, 0., 'real*8');
fwrite(FID, 8, 'integer*4');



% we determine and we write the new velocity and vorticity
for iz=1:nz_final
	field_write(:,:,iz) =  VBX + ratio*Velo_X(:,:,1)*cos(KZ*ZZ(iz));
end
fwrite(FID, nx*ny*nz_final*8, 'integer*4');
fwrite(FID, field_write, 'real*8');
fwrite(FID, nx*ny*nz_final*8, 'integer*4');

for iz=1:nz_final
	field_write(:,:,iz) =  VBY + ratio*Velo_Y(:,:,1)*cos(KZ*ZZ(iz));
end
fwrite(FID, nx*ny*nz_final*8, 'integer*4');
fwrite(FID, field_write, 'real*8');
fwrite(FID, nx*ny*nz_final*8, 'integer*4');

for iz=1:nz_final
	field_write(:,:,iz) =  VBZ + ratio*Velo_Z(:,:,1)*sin(-KZ*ZZ(iz));
end
fwrite(FID, nx*ny*nz_final*8, 'integer*4');
fwrite(FID, field_write, 'real*8');
fwrite(FID, nx*ny*nz_final*8, 'integer*4');


% we write a null density perturbation
field_write =  zeros(nx,ny,nz_final);
fwrite(FID, nx*ny*nz_final*8, 'integer*4');
fwrite(FID, field_write, 'real*8');
fwrite(FID, nx*ny*nz_final*8, 'integer*4');

% we write the vorticity
for iz=1:nz_final
	field_write(:,:,iz) =  WBX + ratio*Vort_X(:,:,1)*sin(-KZ*ZZ(iz));
end
fwrite(FID, nx*ny*nz_final*8, 'integer*4');
fwrite(FID, field_write, 'real*8');
fwrite(FID, nx*ny*nz_final*8, 'integer*4');


for iz=1:nz_final
	field_write(:,:,iz) =  WBY + ratio*Vort_Y(:,:,1)*sin(-KZ*ZZ(iz));
end
fwrite(FID, nx*ny*nz_final*8, 'integer*4');
fwrite(FID, field_write, 'real*8');
fwrite(FID, nx*ny*nz_final*8, 'integer*4');


for iz=1:nz_final
	field_write(:,:,iz) =  WBZ + ratio*Vort_Z(:,:,1)*cos(KZ*ZZ(iz));
end
fwrite(FID, nx*ny*nz_final*8, 'integer*4');
fwrite(FID, field_write, 'real*8');
fwrite(FID, nx*ny*nz_final*8, 'integer*4');

% we write a null base state
fwrite(FID, nx*ny*8*3, 'integer*4');
fwrite(FID, VBX, 'real*8');
fwrite(FID, VBY, 'real*8');
fwrite(FID, VBZ, 'real*8');
fwrite(FID, nx*ny*8*3, 'integer*4');

fwrite(FID, nx*ny*8*3, 'integer*4');
fwrite(FID, WBX, 'real*8');
fwrite(FID, WBY, 'real*8');
fwrite(FID, WBZ, 'real*8');
fwrite(FID, nx*ny*8*3, 'integer*4');

status = fclose(FID);

% we release the memory
clear field_write

fprintf(' -> ok\n');
