program main

    use timing
    use bounded_nonlin_LS

    implicit none

    ! dimensions of the data file
    integer, parameter :: nx=128
    integer, parameter :: ny=128
    integer, parameter :: nz=64

    ! maximum number of data files
    integer, parameter :: nb_file_max=200
    character(len=200) :: nom_fichier
    integer :: nb_file
    character(len=200), dimension(nb_file_max) :: file_name
    double precision, dimension(nb_file_max) :: instants

    ! slices of data
    double precision, dimension(0:nx-1,0:ny-1) :: wz_iz

    ! fitting results and relative error
    integer, parameter :: nb_adjacent=1
    integer :: type_fitting=1
    character(len=10) :: text_type_fitting
    integer, parameter :: n_max = 6
    integer :: n_fitting
    double precision, dimension(nb_file_max, 0:nz-1, n_max) :: param_vortex_p
    double precision, dimension(nb_file_max, 0:nz-1, n_max) :: param_vortex_n
    double precision, dimension(nb_file_max, 0:nz-1) :: erreur_rel
    integer, dimension(nb_file_max, 0:nz-1) :: code_convergence
    double precision, dimension(n_max) :: bl_p, bu_p, bl_n, bu_n

    !: colocation points in the x- and y- directions
    double precision, dimension(0:nx-1) :: xx
    double precision, dimension(0:ny-1) :: yy

    ! physical dimensions of the box
    double precision :: lx,ly,lz

    integer :: eof
    integer :: i_file, ix,iy,iz

    double precision :: pi


    pi = acos(-1.)

    write(*,*) '************************************'
    write(*,*) '******  PARAM VORTEX FITTING  ******'
    write(*,*) '************************************'

    write(*,*)
    write(*,*) 'Least-Squares library used: elsunc'
    write(*,*)

    ! On lit les noms des fichiers a traiter
    write(*,*) 'Fichier contenant les noms des fichiers a analyser :'
    read(*,*) nom_fichier
    write(*,*)

    open (unit=8,file=nom_fichier,form="formatted",status="old",action="read",position="rewind",iostat=eof)
    nb_file = 0
    do while (eof==0)
        nb_file = nb_file + 1
        read(8,*,iostat=eof)  file_name(nb_file)
    end do
    close(8)

    nb_file = nb_file - 1

    do i_file=1, nb_file
        write(*,*) '    ', trim(file_name(i_file))
    end do

    write(*,*)
    write(*,*) '   --> Number of data files: ', nb_file


    ! on lit les meta donnees dans le premier fichier (dimensions de la boite...)
    call read_metadata(file_name(1))

    write(*,*)
    write(*,*) 'Number of points: ', nx, ny, nz
    write(*,*) 'Dimensions: ', lx, ly, lz
    write(*,*)

    ! on initialise les points de grille
    do ix=0,nx-1
      xx(ix) = ix*lx/nx
    end do

    do iy=0,ny-1
      yy(iy) = iy*ly/ny
    end do

    ! Fitting parameters: x_p, y_p, Gamma_p, a_p
    if (type_fitting==1) then
        n_fitting = 4
        text_type_fitting='1'

        bl_p(1:n_fitting) = (/-30., -30., 0., 0.05/)
        bu_p(1:n_fitting) = (/30., 30., 8., 3./)

        bl_n(1:n_fitting) = (/-30., -30.,-8.,0.05/)
        bu_n(1:n_fitting) = (/30., 30., 0., 3./)

        ! initialization of the guess value
        do iz=0,nz-1
            do i_file=1,nb_file
                param_vortex_p(i_file,iz,1:n_fitting) = (/15., 23., 6.28,1./)
                param_vortex_n(i_file,iz,1:n_fitting) = (/15., 15., -6.28,1./)
            end do
        end do

    ! Fitting parameters: x_p, y_p, Gamma_p, a_p, b_p
    else if (type_fitting==2) then
        n_fitting=5
        text_type_fitting='2'

        bl_p(1:n_fitting) = (/-10., -10., 0.,0.05,0.05/)
        bu_p(1:n_fitting) = (/10., 10., 2.2,3,3/)

        bl_n(1:n_fitting) = (/-10., -10., -2.2,0.05,0.05/)
        bu_n(1:n_fitting) = (/10., 10., 0.,3.,3./)

        ! initialization of the guess value
        do iz=0,nz-1
            do i_file=1,nb_file
                param_vortex_p(i_file,iz,1:n_fitting) = (/2.1, 2.35, 2.,0.2, 0.2/)
                param_vortex_n(i_file,iz,1:n_fitting) = (/2.1, 1.85, -2.,0.2, 0.2/)
            end do
        end do

    ! Fitting parameters: x_p, y_p, Gamma_p, a_p, b_p, theta_p
    else if (type_fitting==3) then
        n_fitting=6
        text_type_fitting='3'

        ! x_p, y_p, Gamma_p, a_p, b_p, theta_p
        bl_p(1:n_fitting) = (/-10., -10., 0.,0.05,0.05,-3.14/2./)
        bu_p(1:n_fitting) = (/10., 10., 2.2,3.,3.,3.14/2./)

        bl_n(1:n_fitting) = (/-10., -10., -2.2,0.05,0.05,-3.14/2./)
        bu_n(1:n_fitting) = (/10., 10., 0.,3.,3.,3.14/2./)

        ! initialization of the guess value
        do iz=0,nz-1
            do i_file=1,nb_file
                param_vortex_p(i_file,iz,1:n_fitting) = (/2.1, 2.35, 2.,0.2, 0.2, 0./)
                param_vortex_n(i_file,iz,1:n_fitting) = (/2.1, 1.85, -2.,0.2, 0.2, 0./)
            end do
        end do

    end if



    ! pour chaque fichier et chaque plan horizontal, on determine les parametres fittant au mieux le champ vortical observe
    write(*,*) '------------------------------------------'
    write(*,*) 'Fitting type:', type_fitting
    write(*,*) '------------------------------------------'

    do i_file=1,nb_file

        ! we determine the time of the record
        open (unit=8,file=file_name(i_file),access='direct',form='unformatted',recl=34)
        read(8,rec=2) instants(i_file)
        close(8)
        write(*,*)
        write(*,*) 'time=',instants(i_file)

        do iz=0,nz-1

            ! Fitting of the parameters of the vortices
            call timer_start('fitting_param')

            call read_slice_data(file_name(i_file), iz, wz_iz,'wz')

            call get_char_vortices(wz_iz,param_vortex_p(max(i_file-1,1),iz,:),param_vortex_n(max(i_file-1,1),iz,:), &
                                   param_vortex_p(i_file,iz,:),param_vortex_n(i_file,iz,:), erreur_rel(i_file,iz), &
                                   code_convergence(i_file,iz))

            write(*,*) '    iz=',iz, '  Code convergence=',code_convergence(i_file,iz)
            write(*,*) '          x_p=',param_vortex_p(i_file,iz,1)
            write(*,*) '          y_p=',param_vortex_p(i_file,iz,2)
            write(*,*) '          Gamma_p=',param_vortex_p(i_file,iz,3)
            write(*,*) '          a_p=',param_vortex_p(i_file,iz,4)
            if (type_fitting==2 .or. type_fitting==3)  write(*,*) '          b_p=',param_vortex_p(i_file,iz,5)
            if (type_fitting==3)  write(*,*) '          theta_p=',param_vortex_p(i_file,iz,6)
            write(*,*) '          error_rel=',erreur_rel(i_file,iz)

            call timer_stop('fitting_param')
        end do
    end do

    ! On ecrit les resultats
    open (unit=76,file='param_vortex_results_'//trim(nom_fichier)//'_type_fitting='//text_type_fitting,form='unformatted')
    write(76) 1,nx,ny,nz,lx,ly,lz
    write(76) nb_file
    write(76) type_fitting, n_fitting
    write(76) instants(1:nb_file)
    write(76) param_vortex_p(1:nb_file,:,1:n_fitting)
    write(76) param_vortex_n(1:nb_file,:,1:n_fitting)
    write(76) erreur_rel(1:nb_file,:)
    write(76) code_convergence(1:nb_file,:)
    close(76)

    call timer_report_all

    contains



!**************************
    subroutine get_char_vortices(wz_iz,guess_p, guess_n, param_vortex_p, param_vortex_n, erreur_rel_iz, code_convergence_iz)
!**************************

        implicit none

        double precision, dimension(0:nx-1,0:ny-1), intent(in) :: wz_iz
        double precision, dimension(n_max), intent(in) :: guess_p, guess_n
        double precision, dimension(n_max), intent(out) :: param_vortex_p, param_vortex_n
        double precision, intent(out) :: erreur_rel_iz
        integer, intent(out) :: code_convergence_iz

        double precision, dimension(0:nx-1,0:ny-1), save :: erreur_abs
        double precision, dimension(nx*ny,2*n_max), save  :: c
        integer, dimension(11), save :: p = -1
        double precision, dimension(6), save :: w = -1.

        double precision, dimension(2*n_max) :: param_vortex, bl, bu

		double precision :: somme1,somme2
        integer :: eof

        param_vortex(1:n_fitting)             = guess_p(1:n_fitting)
        param_vortex(n_fitting+1:2*n_fitting) = guess_n(1:n_fitting)

        bl(1:n_fitting)               = bl_p(1:n_fitting)
        bl(n_fitting+1:2*n_fitting)   = bl_n(1:n_fitting)
        bu(1:n_fitting)               = bu_p(1:n_fitting)
        bu(n_fitting+1:2*n_fitting)   = bu_n(1:n_fitting)

        ! on appelle la fonction de moindre carres
        call elsunc(param_vortex(1:2*n_fitting), 2*n_fitting, nx*ny, nx*ny, fcn, 2, bl, bu, p, w, code_convergence_iz, erreur_abs, c)

        param_vortex_p(1:n_fitting) = param_vortex(1:n_fitting)
        param_vortex_n(1:n_fitting) = param_vortex(n_fitting+1:2*n_fitting)

        ! on calcule l'erreur relative totale et on l'enregistre
        somme1 = 0.
        somme2 = 0.
        do iy=0, ny-1
          do ix=0, nx-1
            if (wz_iz(ix,iy)>0) then
                somme1 = somme1 + wz_iz(ix,iy)**2
                somme2 = somme2 + erreur_abs(ix,iy)**2
            end if
          end do
        end do

        erreur_rel_iz = somme2/somme1

end subroutine



!**************************
    subroutine fcn(param_vortex,n,diff_theorie_obs,m,ctrl,c,mdc)
!**************************

        implicit none

        integer, intent(in) :: m,n,mdc
        integer, intent(inout) :: ctrl
        double precision, dimension(n), intent(in) :: param_vortex
        double precision, dimension(0:nx-1,0:ny-1), intent(out) :: diff_theorie_obs
        double precision, dimension(mdc,n), intent(in) :: c

        double precision, dimension(0:nx-1,0:ny-1), save :: vort_theorique

        double precision :: x_p, y_p, Gamma_p, a_p, b_p, theta_p, c_p2, xx_p2, yy_p2
        double precision :: x_n, y_n, Gamma_n, a_n, b_n, theta_n, c_n2, xx_n2, yy_n2
        integer :: ix_box, iy_box, ix, iy
        double precision :: trash

        ! evite un avertissement donne par certains compilateurs
        trash=m+n+mdc+c(1,1)

        ! control parameter to choose what should be computed
        if (ctrl==2) then
            ctrl=0
            return
        end if


        if (type_fitting==1) then
            x_p = modulo(param_vortex(1),lx)
            y_p = modulo(param_vortex(2),ly)
            Gamma_p = param_vortex(3)
            a_p = param_vortex(4)

            x_n = modulo(param_vortex(5),lx)
            y_n = modulo(param_vortex(6),ly)
            Gamma_n = param_vortex(7)
            a_n = param_vortex(8)

            c_p2 = a_p**2
            c_n2 = a_n**2

            vort_theorique = 0.
            do iy_box=-nb_adjacent,nb_adjacent
               do ix_box=-nb_adjacent,nb_adjacent
                  do iy=0, ny-1
                    do ix=0, nx-1
                        xx_p2 = (xx(ix) - x_p-ix_box*lx)**2/a_p**2
	    				yy_p2 = (yy(iy) - y_p-iy_box*ly)**2/a_p**2

                        xx_n2 = (xx(ix) - x_n-ix_box*lx)**2/a_n**2
	    				yy_n2 = (yy(iy) - y_n-iy_box*ly)**2/a_n**2

                        vort_theorique(ix,iy) = vort_theorique(ix,iy) + Gamma_p/(pi*c_p2)*exp(-xx_p2-yy_p2) + Gamma_n/(pi*c_n2)*exp(-xx_n2-yy_n2)
                    end do
                  end do
               end do
             end do


        else if (type_fitting==2) then
            x_p = modulo(param_vortex(1),lx)
            y_p = modulo(param_vortex(2),ly)
            Gamma_p = param_vortex(3)
            a_p = param_vortex(4)
            b_p = param_vortex(5)

            x_n = modulo(param_vortex(6),lx)
            y_n = modulo(param_vortex(7),ly)
            Gamma_n = param_vortex(8)
            a_n = param_vortex(9)
            b_n = param_vortex(10)

            c_p2 = 0.5*(a_p**2+b_p**2)
            c_n2 = 0.5*(a_n**2+b_n**2)

            vort_theorique = 0.
            do iy_box=-nb_adjacent,nb_adjacent
               do ix_box=-nb_adjacent,nb_adjacent
                  do iy=0, ny-1
                    do ix=0, nx-1
                        xx_p2 = (xx(ix) - x_p-ix_box*lx)**2/a_p**2
	    				yy_p2 = (yy(iy) - y_p-iy_box*ly)**2/b_p**2

                        xx_n2 = (xx(ix) - x_n-ix_box*lx)**2/a_n**2
	    				yy_n2 = (yy(iy) - y_n-iy_box*ly)**2/b_n**2

                        vort_theorique(ix,iy) = vort_theorique(ix,iy) + Gamma_p/(pi*c_p2)*exp(-xx_p2-yy_p2) + Gamma_n/(pi*c_n2)*exp(-xx_n2-yy_n2)
                    end do
                  end do
               end do
             end do


        else if (type_fitting==3) then
            x_p = modulo(param_vortex(1),lx)
            y_p = modulo(param_vortex(2),ly)
            Gamma_p = param_vortex(3)
            a_p = param_vortex(4)
            b_p = param_vortex(5)
            theta_p = param_vortex(6)

            x_n = modulo(param_vortex(7),lx)
            y_n = modulo(param_vortex(8),ly)
            Gamma_n = param_vortex(9)
            a_n = param_vortex(10)
            b_n = param_vortex(11)
            theta_n = param_vortex(12)

            c_p2 = 0.5*(a_p**2+b_p**2)
            c_n2 = 0.5*(a_n**2+b_n**2)

            vort_theorique = 0.
            do iy_box=-nb_adjacent,nb_adjacent
               do ix_box=-nb_adjacent,nb_adjacent
                  do iy=0, ny-1
                    do ix=0, nx-1
                        xx_p2 = (  (xx(ix) - x_p-ix_box*lx)*cos(theta_p) + (yy(iy) - y_p-iy_box*ly)*sin(theta_p) )**2/a_p**2
	    				yy_p2 = ( -(xx(ix) - x_p-ix_box*lx)*sin(theta_p) + (yy(iy) - y_p-iy_box*ly)*cos(theta_p) )**2/b_p**2

                        xx_n2 = (  (xx(ix) - x_n-ix_box*lx)*cos(theta_n) + (yy(iy) - y_n-iy_box*ly)*sin(theta_n) )**2/a_n**2
    					yy_n2 = ( -(xx(ix) - x_n-ix_box*lx)*sin(theta_n) + (yy(iy) - y_n-iy_box*ly)*cos(theta_n) )**2/b_n**2

                        vort_theorique(ix,iy) = vort_theorique(ix,iy) + Gamma_p/(pi*c_p2)*exp(-xx_p2-yy_p2) + Gamma_n/(pi*c_n2)*exp(-xx_n2-yy_n2)
                    end do
                  end do
               end do
             end do

        end if

        ! on calcule l'ecart a la vorticite observee
        do iy=0, ny-1
          do ix=0, nx-1
                diff_theorie_obs(ix,iy) = vort_theorique(ix,iy)-wz_iz(ix,iy)
          end do
        end do

end subroutine


!**************************
    subroutine read_metadata(file_name)
!**************************
!    we read the metadata and we check the dimensions in the file

        implicit none

        character(len=200), intent(in) :: file_name
        integer :: flag, nxread, nyread, nzread

        ! we check the dimensions
        open (unit=8,file=file_name,form='unformatted')
        read(8) flag,nxread,nyread,nzread,lx,ly,lz
        close(8)
        if (flag /= 1 .or. nxread /= nx .or. nyread /= ny .or. nzread /= nz) then
            write(*,*) "ERROR: the data file has bad format. Failure."
            stop
        end if

    end subroutine



!**************************
    subroutine read_slice_data(file_name,iz, data_field,name_field)
!**************************

        implicit none

        character(len=200), intent(in) :: file_name
        integer, intent(in) :: iz
        double precision, dimension(0:nx-1,0:ny-1),intent(out) :: data_field
        character(len=2), intent(in) :: name_field
        integer :: position_record

        if (name_field == 'vx')  then
            position_record=38+0*2*(1+nx*ny*nz)+2*iz*nx*ny
        else if (name_field == 'vy') then
            position_record=38+1*2*(1+nx*ny*nz)+2*iz*nx*ny
        else if (name_field == 'vz') then
            position_record=38+2*2*(1+nx*ny*nz)+2*iz*nx*ny
        else if (name_field == 'rho') then
            position_record=38+3*2*(1+nx*ny*nz)+2*iz*nx*ny
        else if (name_field == 'wx') then
            position_record=38+4*2*(1+nx*ny*nz)+2*iz*nx*ny
        else if (name_field == 'wy') then
            position_record=38+5*2*(1+nx*ny*nz)+2*iz*nx*ny
        else if (name_field == 'wz') then
            position_record=38+6*2*(1+nx*ny*nz)+2*iz*nx*ny
        end if

        ! we open the data file again and we go directly to the data slice of interest
        open (unit=8,file=file_name,access='direct',form='unformatted',recl=position_record)
        read(8,rec=2) data_field
        close(8)

    end subroutine

end program
