program main

  use timing

  implicit none

  integer, parameter :: nx=128
  integer, parameter :: ny=128
  integer, parameter :: nz=64
  integer, parameter :: nb_fichiers_max=200
  integer, parameter :: nb_adjacent=1

  double precision, dimension(0:nx-1,0:ny-1) :: vort_iz
  character(len=200), dimension(nb_fichiers_max) :: file_name
  character(len=200) :: nom_fichier
  double precision, dimension(nb_fichiers_max) :: instants
  integer :: nb_fichiers
  integer :: eof
  integer :: i_time, iz, ix, iy
  double precision, dimension(0:nx-1) :: xx
  double precision, dimension(0:ny-1) :: yy
  double precision :: lx,ly,lz
  double precision, dimension(nb_fichiers_max, nz, 9) :: char_vortices

	write(*,*) 'Least-Squares library used: minpack'
	write(*,*)

	! On lit les noms des fichiers a traiter
	write(*,*) 'Fichier contenant les noms des fichiers a analyser :'
	read(*,*) nom_fichier
	write(*,*)

	open (unit=8,file=nom_fichier,form="formatted",status="old",action="read",position="rewind",iostat=eof)
	nb_fichiers = 0
	do while (eof==0)
		nb_fichiers = nb_fichiers + 1
		read(8,*,iostat=eof)  file_name(nb_fichiers)
		write(*,*) '    ', trim(file_name(nb_fichiers))
	end do
	close(8)

	nb_fichiers = nb_fichiers - 1

	write(*,*)
	write(*,*) '   ---> Nombre de fichiers : ', nb_fichiers



  ! on lit les meta donnees dans le premier fichier (dimensions de la boite...)
  call read_metadata(file_name(1))

  write(*,*)
  write(*,*) 'Number of points: ', nx, ny, nz
  write(*,*) 'Dimensions: ', lx, ly, lz
  write(*,*)

  ! on initialise les points de grille
  do ix=0,nx-1
      xx(ix) = ix*lx/nx
  end do

  do iy=0,ny-1
      yy(iy) = iy*ly/ny
  end do

  ! on initialise la guess-value
  do iz=0,nz-1
    char_vortices(1,iz,:) = (/2.1,2.,2.,0.2,2.1,2.,-2.,0.2,0./)
  end do


! pour chaque fichier et chaque plan horizontal, on determine les parametres fittant au mieux le champ vortical observe
  do i_time=1,nb_fichiers

		! we determine the time of the record
		open (unit=8,file=file_name(i_time),access='direct',form='unformatted',recl=34)
		read(8,rec=2) instants(i_time)
		close(8)
    write(*,*)
		write(*,*) 'time=',instants(i_time)

		do iz=9,9
			call timer_start('loop')
      call read_vort_iz(file_name(i_time), iz, vort_iz)
      call get_char_vortices(vort_iz,char_vortices(max(i_time-1,1),iz,:),char_vortices(i_time,iz,:))
      write(*,*) '    iz=',iz, '  error=',char_vortices(i_time,iz,9)
      write(*,*) '          x_p=',char_vortices(i_time,iz,1)
      write(*,*) '          y_p=',char_vortices(i_time,iz,2)
      write(*,*) '          Gamma_p=',char_vortices(i_time,iz,3)
      write(*,*) '          a_p=',char_vortices(i_time,iz,4)
      call timer_stop('loop')
		end do
  end do

  ! On ecrit les resultats
  open (unit=76,file='param_vortex_results_'//nom_fichier,form='unformatted')
  write(76) nx,ny,nz,lx,ly,lz
  do i_time=1,nb_fichiers
  	write(76) instants(i_time), char_vortices(i_time,:,:)
  end do
  close(76)

  call timer_report_all

  contains



!**************************
  subroutine get_char_vortices(vort_iz,guess_param_vortex, param_vortex)
!**************************

    implicit none

    double precision, dimension(0:nx-1,0:ny-1), intent(in) :: vort_iz
    double precision, dimension(9), intent(in) :: guess_param_vortex
    double precision, dimension(9), intent(out) :: param_vortex

    integer :: ix, iy

    integer, parameter :: n=8
    double precision, dimension(0:nx-1,0:ny-1), save :: erreur_abs
    integer :: info
    integer, dimension(n) :: iwa
    integer, parameter :: lwa = nx*ny*n+5*n+nx*ny
    double precision, dimension(lwa) :: wa
    double precision :: somme1,somme2

    param_vortex(1:8) = guess_param_vortex(1:8)

    ! on appelle la fonction de moindre carres
    call lmdif1(fcn,nx*ny,8,param_vortex(1:8),erreur_abs,0.1,info,iwa,wa,lwa)

    ! on calcule l'erreur relative totale et on l'enregistre
    somme1 = 0.
    somme2 = 0.
    do iy=0, ny-1
      do ix=0, nx-1
        somme1 = somme1 + vort_iz(ix,iy)**2
        somme2 = somme2 + erreur_abs(ix,iy)**2
      end do
    end do

    param_vortex(9) = somme2/somme1

  end subroutine



!**************************
    subroutine fcn(m,n,param_vortex,diff_theorie_obs,iflag)
!**************************

    implicit none

    integer, intent(in) :: m,n
    integer, intent(inout) :: iflag
    double precision, dimension(n), intent(in) :: param_vortex
    double precision, dimension(0:nx-1,0:ny-1), intent(out) :: diff_theorie_obs

    double precision, dimension(0:nx-1,0:ny-1), save :: vort_theorique
    double precision, dimension(0:nx-1,0:ny-1, 2*nb_adjacent+1, 2*nb_adjacent+1), save :: distance_p, distance_n

    double precision :: x_p, y_p, x_n, y_n, Gamma_p, a_p, Gamma_n, a_n
    double precision :: pi
    integer :: ix_box, iy_box, ix, iy
    integer :: trash

    ! evite un avertissement donne par certains compilateurs
    trash=m+n

    pi = acos(-1.)

    ! on recupere les parametres des deux vortex de Lamb-Oseen
    ! pour les positions des deux vortex, on fait attention a la periodicite de la boite...
    x_p = modulo(param_vortex(1),lx)
    y_p = modulo(param_vortex(2),ly)
    Gamma_p = param_vortex(3)
    a_p = param_vortex(4)

    x_n = modulo(param_vortex(5),lx)
    y_n = modulo(param_vortex(6),ly)
    Gamma_n = param_vortex(7)
    a_n = param_vortex(8)

    ! on calcule la distance des differents vortex aux points de grille
    do iy_box=-nb_adjacent,nb_adjacent
       do ix_box=-nb_adjacent,nb_adjacent
          do iy=0, ny-1
            do ix=0, nx-1
              distance_p(ix,iy,ix_box+nb_adjacent+1,iy_box+nb_adjacent+1) = (xx(ix) - x_p-ix_box*lx)**2 + (yy(iy) - y_p-iy_box*ly)**2;
              distance_n(ix,iy,ix_box+nb_adjacent+1,iy_box+nb_adjacent+1) = (xx(ix) - x_n-ix_box*lx)**2 + (yy(iy) - y_n-iy_box*ly)**2;
            end do
          end do
       end do
     end do

		! on calcule la vorticite theorique correspondante
    vort_theorique = 0.
    do iy_box=1,2*nb_adjacent+1
      do ix_box=1,2*nb_adjacent+1
        do iy=0, ny-1
          do ix=0, nx-1
            vort_theorique(ix,iy) = vort_theorique(ix,iy) + Gamma_p/(pi*a_p**2)*exp(-distance_p(ix,iy,ix_box,iy_box)/a_p**2) + Gamma_n/(pi*a_n**2)*exp(-distance_n(ix,iy,ix_box,iy_box)/a_n**2);
          end do
        end do
      end do
    end do

    ! on calcule l'ecart a la vorticite observee
    do iy=0, ny-1
      do ix=0, nx-1
        diff_theorie_obs(ix,iy) = vort_theorique(ix,iy)-vort_iz(ix,iy)
      end do
    end do

  end subroutine


!**************************
  subroutine read_metadata(file_name)
!**************************
!    we read the metadata and we check the dimensions in the file

        implicit none

        character(len=200), intent(in) :: file_name
        integer :: flag, nxread, nyread, nzread

        ! we check the dimensions
        open (unit=8,file=file_name,form='unformatted')
        read(8) flag,nxread,nyread,nzread,lx,ly,lz
        close(8)
        if (flag /= 1 .or. nxread /= nx .or. nyread /= ny .or. nzread /= nz) then
            write(*,*) "ERROR: the data file has bad format. Failure."
            stop
        end if

    end subroutine



!**************************
    subroutine read_vort_iz(file_name,iz, vort_iz)
!**************************

        implicit none

        character(len=200), intent(in) :: file_name
        integer, intent(in) :: iz
        double precision, dimension(0:nx-1,0:ny-1),intent(out) :: vort_iz

    ! we open the data file again and we go directly to the data slice of interest
    open (unit=8,file=file_name,access='direct',form='unformatted',recl=38+6*2*(1+nx*ny*nz)+2*iz*nx*ny)
    read(8,rec=2) vort_iz
        close(8)

  end subroutine

end program
