% ######################
% global_vars a renseigner
% ######################

% base directory
dir_base ='.';

% choice of what we plot
%   1) the global mode at the last instant
%   2) the mode at all the instants
%   3) we plot the mode at the specific instants indicated below
mode_affichage = 2;

% choice of plotting matlab function
%   1) function contour. Nice but long for complicated plot.
%   2) function pcolor. Quick but rough.
type_graphique = 2;

% number of plots per figure
subplot_par_figure = [4,3];


% ######################
% on touche plus
% ######################

fprintf(1, '\nBase directory: %s\n\n', dir_base);

if strcmp('PCWIN', computer)
	system(['dir /O-D /AD /B ', dir_base]);
else
	system(['ls -t ', dir_base]);
end

dir_spec = input('\nSpecific directory: ', 's');

dir_complet = [dir_base,'/',dir_spec,'/'];


% noms des fichiers de donnees
if strcmp('PCWIN', computer)
	[poub,liste_fichiers] = system(['dir /B "', dir_complet, '"']);
else
	[poub,liste_fichiers] = system(['ls ', dir_complet]);
end

[start,finish] = regexpi(liste_fichiers,'velo_rho_vort.t=\d*\.\d*');

nb_fichiers = size(start,2);

clear nom_fichier
for i=1:nb_fichiers
    nom_fichier(i,:) = liste_fichiers(start(i):finish(i));
end

nom_fichier = sortrows(nom_fichier);
fichier = nom_fichier(1,:)

% we determine the type of binary data (little-endian, big-endian) with the file header

% we start with little-endian (PC)
type_endian = 'ieee-le';
FID = fopen(fichier, 'r',type_endian);
poub = fread(FID, 1, 'integer*4');
flag = fread(FID, 1, 'integer*4');
status = fclose(FID);

% if the flag is not 1, we try big-endian (SGI, NEC, IBM)
if flag ~= 1
    type_endian = 'ieee-be';
    FID = fopen(fichier, 'r',type_endian);
    poub = fread(FID, 1, 'integer*4');
    flag = fread(FID, 1, 'integer*4');
    status = fclose(FID);

    % we check again the value of the flag
    if flag ~= 1
        fprintf(1, '\n There is a problem with the format of the data file: the flag is not 1\n');
        return
    end
end


% we read the metadata
FID = fopen(fichier, 'r',type_endian);

poub            = fread(FID, 1, 'integer*4');
flag            = fread(FID, 1, 'integer*4');
nx              = fread(FID, 1, 'integer*4');
ny              = fread(FID, 1, 'integer*4');
nz              = fread(FID, 1, 'integer*4');
lx              = fread(FID, 1, 'real*8');
ly              = fread(FID, 1, 'real*8');
lz              = fread(FID, 1, 'real*8');
dt              = fread(FID, 1, 'real*8');
truncate        = fread(FID, 1, 'integer*4');
type_trunc      = fread(FID, 1, 'integer*4');
rtrunc_x        = fread(FID, 1, 'real*8');
rtrunc_y        = fread(FID, 1, 'real*8');
rtrunc_z        = fread(FID, 1, 'real*8');
nu              = fread(FID, 1, 'real*8');
stratification  = fread(FID, 1, 'integer*4');
xns             = fread(FID, 1, 'real*8');
schm            = fread(FID, 1, 'real*8');
omega2          = fread(FID, 1, 'real*8');
perturb        = fread(FID, 1, 'integer*4');
lin             = fread(FID, 1, 'integer*4');
poub            = fread(FID, 1, 'integer*4');

status = fclose(FID);

% we display some informations about the simulation
fprintf(1, '\n Dimensions of the simulation: %i * %i * %i \n', nx, ny, nz);
fprintf(1, '\n Physical dimensions of the box: %f * %f * %f \n', lx, ly, lz);

if (stratification == 0)
    freq_brunt = 0;
else
    freq_brunt = xns;
end

fprintf(1, '\n Stratification: %f \n', freq_brunt);

if (perturb==0)
    texte = ['non perturbative'];
else
    if (lin==0) then
        texte = ['perturbative/non linear'];
    else
        texte = ['perturbative/linear'];
    end
end

% on cre la grille des points de colocation
X = linspace(0, lx-lx/nx, nx);
Y = linspace(0, ly-ly/ny, ny);
Z = linspace(0, lz-lz/nz, nz);

% on determine selon quel plan reprsente le champ vectoriel 3D
NX_plot = 1;
NY_plot = 1;
NZ_plot = 1;


if mode_affichage==1

    % on lit les donnees dans le dernier fichier enregistre
    FID = fopen([dir_complet,'/',nom_fichier(nb_fichiers,:)], 'r',type_endian);

    % on lit toutes les metadonnees d'un coup "comme si" c'tait des entiers de 4 octets mme si ce n'est pas vrai,
    % l'important est que le nombre d'octets soit respecte
    poub = fread(FID, 33, 'integer*4');

    poub = fread(FID, 1, 'integer*4');
    instant = fread(FID, 1, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Velo_X = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Velo_Y = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Velo_Z = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Rho = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Vort_X = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Vort_Y = fread(FID, nx*ny*nz, 'real*8');
    poub = fread(FID, 2, 'integer*4');
    Vort_Z = fread(FID, nx*ny*nz, 'real*8');

    status = fclose(FID);

    Velo_X = reshape(Velo_X, [nx,ny,nz]);
    Velo_Y = reshape(Velo_Y, [nx,ny,nz]);
    Velo_Z = reshape(Velo_Z, [nx,ny,nz]);
    Rho    = reshape(Rho,    [nx,ny,nz]);
    Vort_X = reshape(Vort_X, [nx,ny,nz]);
    Vort_Y = reshape(Vort_Y, [nx,ny,nz]);
    Vort_Z = reshape(Vort_Z, [nx,ny,nz]);

    figure
    if type_graphique==1
        contour(X,Y,squeeze(Vort_Z(:,:,NZ_plot))',20);
        colorbar;
    else
        pcolor(X,Y,squeeze(Vort_Z(:,:,NZ_plot))');
        shading flat,
    end

    xlabel('x');
    ylabel('y');
    axis square;
    title({['Vorticite verticale  t=', num2str(instant)]; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');

elseif mode_affichage==2

    % we plot the different statistics
    nb_graphiques = nb_fichiers;

    Nb_plot_par_figure = subplot_par_figure(1)*subplot_par_figure(2);
    nb_figure = ceil(nb_graphiques/Nb_plot_par_figure)

    for ind_figure = 1:nb_figure
        figure

       for it=(ind_figure-1)*Nb_plot_par_figure+1:min(ind_figure*Nb_plot_par_figure, nb_graphiques)
            subplot(subplot_par_figure(1), subplot_par_figure(2), it - (ind_figure-1)*Nb_plot_par_figure );

            % on lit les donnees dans le dernier fichier enregistre
            FID = fopen([dir_complet,'/',nom_fichier(it,:)], 'r',type_endian);

            % on lit toutes les metadonnees d'un coup "comme si" c'tait des entiers de 4 octets mme si ce n'est pas vrai,
            % l'important est que le nombre d'octets soit respecte
            poub = fread(FID, 33, 'integer*4');

            poub = fread(FID, 1, 'integer*4');
            instant = fread(FID, 1, 'real*8');
            poub = fread(FID, 2, 'integer*4');
            Velo_X = fread(FID, nx*ny*nz, 'real*8');
            poub = fread(FID, 2, 'integer*4');
            Velo_Y = fread(FID, nx*ny*nz, 'real*8');
            poub = fread(FID, 2, 'integer*4');
            Velo_Z = fread(FID, nx*ny*nz, 'real*8');
            poub = fread(FID, 2, 'integer*4');
            Rho = fread(FID, nx*ny*nz, 'real*8');
            poub = fread(FID, 2, 'integer*4');
            Vort_X = fread(FID, nx*ny*nz, 'real*8');
            poub = fread(FID, 2, 'integer*4');
            Vort_Y = fread(FID, nx*ny*nz, 'real*8');
            poub = fread(FID, 2, 'integer*4');
            Vort_Z = fread(FID, nx*ny*nz, 'real*8');

            status = fclose(FID);

            Velo_X = reshape(Velo_X, [nx,ny,nz]);
            Velo_Y = reshape(Velo_Y, [nx,ny,nz]);
            Velo_Z = reshape(Velo_Z, [nx,ny,nz]);
            Rho    = reshape(Rho,    [nx,ny,nz]);
            Vort_X = reshape(Vort_X, [nx,ny,nz]);
            Vort_Y = reshape(Vort_Y, [nx,ny,nz]);
            Vort_Z = reshape(Vort_Z, [nx,ny,nz]);

            if type_graphique==1
                contour(X,Y,squeeze(Vort_Z(:,:,NZ_plot))',20);
                colorbar;
            else
                pcolor(X,Y,squeeze(Vort_Z(:,:,NZ_plot))');
                %pcolor(X,Z,squeeze(Vort_Z(:,ny/2,:))');
                shading flat,
            end

            xlabel('x');
            ylabel('y');
            axis square;
            title({['Vorticite verticale  t=', num2str(instant)]; ['Nb_points=', num2str(nx), 'x', num2str(ny), 'x', num2str(nz), '  L=', num2str(lx), 'x', num2str(ly), 'x', num2str(lz), '  dt=',num2str(dt), '  nu=',num2str(nu),'  N=', num2str(freq_brunt), '  ', texte]},'Interpreter','none');

        end
    end
end
